package de.hdm_stuttgart.mi.sd1.test.task1;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.task1.model.Employee;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;

import static de.hdm_stuttgart.mi.sd1.task1._4_Salary.getSalary;
import static de.hdm_stuttgart.mi.sd1.task1._4_Salary.setSalaries;

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
public class Test_4_Salary extends ExaminationTestDefaults {

    @Test @Marking(points = 6)
    public void test_100() {
        Assert.assertEquals(2000, getSalary(0, 0));
    }

    @Test @Marking(points = 1)
    public void test_500_empty() {
        setSalaries(30000, new Employee[0]);
        setSalaries(0, new Employee[0]);
    }

    @Test @Marking(points = 2)
    public void test_520_single() {
        Employee[] employees = {new Employee(2,10)};
        Assert.assertEquals(2900, employees[0].salary);

        {// Companies payment limit is well above sum of all salaries
            setSalaries(3000, employees);
            Assert.assertEquals(2900, employees[0].salary);
        }

        {// Companies payment limit matches sum of all salaries
            setSalaries(2900, employees);
            Assert.assertEquals(2900, employees[0].salary);
        }

        for (short i = 0; i < 2900; i++) {// Companies payment limit beneath sum of all salaries
            setSalaries( i, employees);
            Assert.assertEquals("Payment limit " + i + ":", i, employees[0].salary);
        }
    }

    @Test
    public void test_540_standard() {

        final Employee[] employees = new Employee[10];
        for (byte i = 0; i < employees.length; i++) {
            employees[i] = new Employee(0, 0);
        }

        {// Companies payment limit is well above sum of all salaries
            setSalaries(30000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.salary);
            }
        }

        {// Companies payment limit matches sum of all salaries
            setSalaries(20000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.salary);
            }
        }

        {// Companies payment limit slightly below sum of all salaries.
            setSalaries(19999, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(2000, e.salary);
            }
        }

        {// Companies payment limit just half of all salaries' sum
            setSalaries(10000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(1000, e.salary);
            }
        }

        {// Companies payment limit just quarter of all salaries' sum
            setSalaries(5000, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(500, e.salary);
            }
        }
        {// Bad year: No salaries
            setSalaries(0, employees);
            for (final Employee e : employees) {
                Assert.assertEquals(0, e.salary);
            }
        }
    }

    @Test
    public void test_600_mixed() {

        final Employee[] employees = new Employee[10];

        for (byte i = 0; i < employees.length; i++) {
            employees[i] = new Employee(i, 2 * i);
        }

        {// Companies payment limit is well above sum of all salaries
            setSalaries(40000, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 2000 + 300 * i, employees[i].salary);
            }
        }

        {// Companies payment limit matches sum of all salaries
            setSalaries(33500, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 2000 + 300 * i, employees[i].salary);
            }
        }

        {// Companies payment limit just half of sum of all salaries
            setSalaries(33500 / 2, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 1000 + 150 * i, employees[i].salary);
            }
        }

        {// Companies payment limit just 1% of sum of all salaries
            setSalaries(33500 / 100, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 20 + 3 * i, employees[i].salary);
            }
        }

        {// Bad year: No salaries
            setSalaries(0, employees);
            for (byte i = 0; i < employees.length; i++) {
                Assert.assertEquals("Employee index " + i + ":", 0, employees[i].salary);
            }
        }
    }
}
