package de.hdm_stuttgart.mi.sd1.task2;

/**
 * <p>Simulating driving a car, draining and charging its battery.</p>
 *
 * <p>An electric car's driving range is being determined by:</p>
 *
 * <ul>
 *     <li>The battery's state of charge measured in kilowatt hours (kwh). A battery
 *     does have a capacity e.g. 50 Kwh. </li>
 *     <li>Its weight and other factors allow for a given number of kilometer per 1 Kwh. </li>
 * </ul>
 *
 * <p>Example: A car delivering 200 km before completely draining a battery of 50 kwh requires
 *    1 Kwh per 4Km of driving. We show some samples using the simulator class.</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Creating a simulator of 50 Kwh capacity and 4 Km per 1 Kwh driving range</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> // fully charged 50 Kwh battery capacity, 4 Km per 1 Kwh driving range
 * BatteryPlanner planner = new BatteryPlanner(50, 4);  // int arguments!
 *
 * // Get battery state of charge category
 * System.out.println(planner.getState());</code></pre>
 *             </td>
 *             <td>
 *                 <pre>HIGH</pre>
 *                 <p>Fully charged, see {@link BatteryState}</p>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>These charging state categories are being defined by the battery's state of charge in relation to its capacity:</p>
 *
 * <ul>
 *     <li> 0 &lt;= percentage &lt; 10% : {@link BatteryState#LOW}</li>
 *     <li> 10 &lt;= percentage &lt; 90% : {@link BatteryState#NORMAL}</li>
 *     <li> 90 &lt;= percentage &lt;= 100% : {@link BatteryState#HIGH}</li>
 * </ul>
 *
 *
 * <p>Driving a car drains the battery:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Driving a fully charged car</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> //...
 * // Drive 150 Km of 50 Km x 4 = 200 Km driving range
 * double actualDistance = planner.drive(150.0);
 * System.out.println("Actual distance: " + actualDistance);
 *
 * // Trying to drive another 200 Km, but only 50 are left by battery power
 * actualDistance = planner.drive(200.0);
 * System.out.println("Actual distance: " + actualDistance);
 *
 * // Battery completely drained
 * System.out.println("Battery state of charge: " + planner.getState());</code></pre>
 *             </td>
 *             <td>
 *                 <pre> Actual distance: 150.0
 * Actual distance: 50.0
 * Battery state of charge: LOW</pre>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 * <p>Now recharging is being required:</p>
 *
 * <table class="goikTableDefaults">
 *     <caption>Charging the battery</caption>
 *     <tbody>
 *         <tr>
 *             <th>Code</th>
 *             <th>Result and explanation</th>
 *         </tr>
 *         <tr>
 *             <td>
 *                 <pre><code class="java"> // ... starting from previous empty battery ...
 * double excess = planner.charge(30.0);
 * System.out.println("Excess:" + excess);
 *
 * excess = planner.charge(50.0);
 * System.out.println("Excess:" + excess);</code></pre>
 *             </td>
 *             <td>
 *                 <pre> Excess:-20.0
 * Excess:30.0</pre>
 *                <ul>
 *                    <li>-20 indicating 20 Kwh lacking to fully charged state.</li>
 *                    <li>30 Kwh indicating that 30 Kwh out of 50 Kwh offered exceed the
 *                    battery's capacity.</li>
 *                </ul>
 *             </td>
 *         </tr>
 *     </tbody>
 * </table>
 *
 */
public class BatteryPlanner {
  // Implement me
}
