package de.hdm_stuttgart.mi.sd1.task2;

import de.hdm_stuttgart.mi.exam.unitmarking.ExaminationTestDefaults;
import de.hdm_stuttgart.mi.exam.unitmarking.Marking;
import de.hdm_stuttgart.mi.sd1.ignore_me.ObjectWrapper;
import org.junit.Assert;
import org.junit.FixMethodOrder;
import org.junit.Test;
import org.junit.runners.MethodSorters;
@FixMethodOrder(MethodSorters.NAME_ASCENDING)

public class TestBatteryPlanner extends ExaminationTestDefaults {

    private final double deltaDistance = 1.E-14;

    /**
     * Create new instance of {@link BatteryPlanner}
     */
    @Test
    @Marking(points = 2)
    public void test_100CreateBatteryPlanner() {
        new ObjectWrapper<>(BatteryPlanner.class,50, 6);
    }

    /**
     *  Check for {@link BatteryState#HIGH}
     * <ol>
     *     <li>Create an instance</li>
     *     <li></li>
     * </ol>
     */
    @Test
    @Marking(points = 2)
    public void test_200() {
        final ObjectWrapper<BatteryPlanner> planner = new ObjectWrapper<>(BatteryPlanner.class,100, 5);

        // New instance is expected to be fully charged
        Assert.assertSame(BatteryState.HIGH, planner.invoke(BatteryState.class, "getState"));
    }

    /**
     * <ol>
     *     <li>Create an instance</li>
     *     <li>Intended driving distance exceeding battery's capacity</li>
     * </ol>
     */
    @Test
    @Marking(points = 2)
    public void test_300() {
        // 100 Kwh and 5 Km per 1 Kwh driving range
        final ObjectWrapper<BatteryPlanner> planner = new ObjectWrapper<>(BatteryPlanner.class,100, 5);

        // Intending 600 Km of driving, reducing state of charge to 0 Kwh, actually driving just 5 x 100 Km = 500 Km
        Assert.assertEquals(500., planner.invoke(double.class, "drive", 600.), deltaDistance);
    }
    /**
     * <ol>
     *     <li>Create an instance</li>
     *     <li>Drive till completely draining battery</li>
     *     <li>Check for {@link BatteryState#LOW}</li>
     * </ol>
     */
    @Test
    @Marking(points = 2)
    public void test_340() {
        // 100 Kwh and 5 Km per 1 Kwh driving range
        final ObjectWrapper<BatteryPlanner> planner = new ObjectWrapper<>(BatteryPlanner.class,100, 5);

        // Fully charged at 100 Kwh
        Assert.assertSame(BatteryState.HIGH, planner.invoke(BatteryState.class, "getState"));

        // Intending 600 Km of driving, reducing state of charge to 0 Kwh, actually driving 5 x 100 Km = 500 Km
        Assert.assertEquals(500., planner.invoke(double.class, "drive", 600.), deltaDistance);
        Assert.assertSame(BatteryState.LOW, planner.invoke(BatteryState.class, "getState"));
    }

    /**
     * <ol>
     *     <li>Create an instance</li>
     *     <li>Drive till completely draining battery</li>
     *     <li>Two step check for complete charging requiring battery capacity amount.</li>
     * </ol>
     */
    @Test
    @Marking(points = 2)
    public void test_360() {
        // 100 Kwh and 5 Km per 1 Kwh driving range
        final ObjectWrapper<BatteryPlanner> planner = new ObjectWrapper<>(BatteryPlanner.class,100, 5);

        // Intending 500 Km of driving draining battery to zero.
        Assert.assertEquals(500., planner.invoke(double.class, "drive", 500.), deltaDistance);
        Assert.assertSame(BatteryState.LOW, planner.invoke(BatteryState.class, "getState"));

        // Battery capacity is 100 Kwh. So charging 60 Kwh still lacks another 40 Kwh towards 100% state of charge.
        Assert.assertEquals(-40., planner.invoke(double.class, "charge", 60.), deltaDistance);

        // Battery now at 60 Kwh will only consume another 40 Kwh leaving us with an excess of 30 Kwh.
        Assert.assertEquals(30., planner.invoke(double.class, "charge", 70.), deltaDistance);
    }

    /**
     * Driving, checking, charging, repeating ...
     */
    @Test
    @Marking(points = 10)
    public void test_800() {
        // 50 Kwh and 4 Km per 1 Kwh driving range
        final ObjectWrapper<BatteryPlanner> planner = new ObjectWrapper<>(BatteryPlanner.class,50, 4);

        // Fully charged at 50 Kwh
        Assert.assertSame(BatteryState.HIGH, planner.invoke(BatteryState.class, "getState"));

        // Charging impossible, battery is full
        Assert.assertEquals(20., planner.invoke(double.class, "charge", 20.), deltaDistance);
        Assert.assertSame(BatteryState.HIGH, planner.invoke(BatteryState.class, "getState"));

        // Driving 100 Km, reducing state of charge to 25 Kwh
        Assert.assertEquals(100., planner.invoke(double.class, "drive", 100.), deltaDistance);
        Assert.assertSame(BatteryState.NORMAL, planner.invoke(BatteryState.class, "getState"));

        // Driving  88 Km, reducing state of charge to 3 Kwh
        Assert.assertEquals(88., planner.invoke(double.class, "drive", 88.), deltaDistance);
        Assert.assertSame(BatteryState.LOW, planner.invoke(BatteryState.class, "getState"));

        // Intending 20 Km of driving, reducing state of charge to 0 Kwh, actually driving 12 Km
        Assert.assertEquals(12., planner.invoke(double.class, "drive", 20.), deltaDistance);
        Assert.assertSame(BatteryState.LOW, planner.invoke(BatteryState.class, "getState"));

        // Charging 20 Kwh, 30 Kwh missing for complete charging.
        Assert.assertEquals(-30., planner.invoke(double.class, "charge", 20.), deltaDistance);
        Assert.assertSame(BatteryState.NORMAL, planner.invoke(BatteryState.class, "getState"));

        // Driving  80 Km, draining battery to zero
        Assert.assertEquals(80., planner.invoke(double.class, "drive", 80.), deltaDistance);
        Assert.assertSame(BatteryState.LOW, planner.invoke(BatteryState.class, "getState"));

        // Charging 60 Kwh, 10 Kwh exceeding capacity.
        Assert.assertEquals(10., planner.invoke(double.class, "charge", 60.), deltaDistance);
        Assert.assertSame(BatteryState.HIGH, planner.invoke(BatteryState.class, "getState"));
    }
}
