package de.hdm_stuttgart.mi.sd1.task1;

/**
 * Payment rules for customer orders.
 */
public class A_Payment {

    /**
     * <p>Calculate order related end customer billing amounts.</p>
     *
     * <p>Due payments are being calculated by the following two rules:</p>
     *
     * <ol>
     *     <li>
     *         <p>For order amounts below 50€ an additional 6.50€ shipping fee applies.</p>
     *     </li>
     *
     *     <li>
     *         <p>Customers paying within two days receive an early payment discount of 2% being applied
     *          to order costs excluding shipping fee.</p>
     *
     *         <p>Examples:</p>
     *          <ul>
     *              <li>
     *                  <p>An order worth of 50.00€ being paid within two days will be reduced by 2% resulting in
     *                     49.00€. The regular costs exceeding 50€ the customer still enjoys a free delivery.</p>
     *              </li>
     *              <li>
     *                  <p>An order worth of 40.00€ being paid within two days will be reduced by 2% resulting in
     *                     39.20€. The regular costs of 40€ not exceeding the 50€ limit adding 6.50€ shipping results in
     *                     a due amount of 45.70€.</p>
     *              </li>
     *          </ul>
     *     </li>
     * </ol>
     *
     * <img src="doc-files/euros.svg" alt="Five euros" style="width: 20em; height: auto;">
     *
     * @param orderAmount The order's sum of all costs
     * @param paymentDelayInDays Period until payment occurs.
     * @return <p>The total due amount a customer is obliged to pay.</p>
     */
    static public double getBillingAmount(final double orderAmount, final int paymentDelayInDays) {

        final double shippingFee = orderAmount < 50 ? 6.50: 0;

        if (paymentDelayInDays <= 2) {
            return orderAmount * 98 / 100 + shippingFee;
        } else {
            return orderAmount + shippingFee;
        }
    }

    private A_Payment(){/* Suppressing constructor Javadoc HTML generation */}
}
